/**
 * Defines the parameters of the system
 **/

#ifndef GLOBAL_H
#define GLOBAL_H

#define NUMBEROFUSERS	5
//#define DEBUG_MODEL 	1

#include <string>
#include <iomanip>	
#include "systemc.h"


/*
 * direction: 1: up, 0: idle, -1: down
 * floor: the current position
 */
typedef struct LiftStatus {
	int direction;
	int floor;
} lift_status_t;


/*
 * type, 0 : internal, 1 : external
 * targetfloor: desired floor, -1 if external
 * fromfloor: external request from, -1 if internal
 * direction: external request direction, 1: up, -1: down, 0 if internal
 * inlift: internal request in the lift inlift, -1 if external
 */

class Request {

public:

	// constructor
	Request (int _type = -1, int _fromfloor = -1, int _targetfloor = -1, int _inlift = -1, int _direction = -1) {
		
		type = _type;
		fromfloor = _fromfloor;
		targetfloor = _targetfloor;
		inlift = _inlift;
		direction = _direction;
	}

	inline bool operator == (const Request& rhs) const {
		return (rhs.type == type && 
			    rhs.fromfloor == fromfloor &&
			    rhs.targetfloor == targetfloor &&
			    rhs.inlift == inlift &&
			    rhs.direction == direction);
	}

	inline Request& operator = (const Request& rhs) {
		type = rhs.type;
		fromfloor = rhs.fromfloor;
		targetfloor = rhs.targetfloor;
		inlift = rhs.inlift;
		direction = rhs.direction;

		return *this;
	}

	inline friend void sc_trace(sc_trace_file *tf, const Request& v, 
		const std::string& NAME) {
		sc_trace(tf, v.type, NAME + ".type");
		sc_trace(tf, v.fromfloor, NAME + ".fromfloor");
		sc_trace(tf, v.targetfloor, NAME + ".targetfloor");
		sc_trace(tf, v.inlift, NAME + ".inlift");
		sc_trace(tf, v.direction, NAME + ".direction");
	}

	inline friend ostream& operator << (ostream& os, Request const & v) {
		os << v.type << " "  << v.fromfloor << " " 
		   << v.targetfloor << " " << v.inlift << " "  
		   << v.direction;
	
		return os;
	}


private:
	int type;
	int fromfloor;
	int targetfloor;
	int inlift;
	int direction;

};

#endif

