/*
 * This module models the behavior of a multi-lift systems.
 * The system consists of NumberofLifts lifts and NumberofFloors floors.
 * Each lift can move a distance of one floor in 0.5s
 */


#ifndef LIFTSYSTEM_H
#define LIFTSYSTEM_H

#include "systemc.h"
#include <tlm.h>
#include <limits.h>
#include "global.h"


SC_MODULE(LiftSystem) {

	SC_HAS_PROCESS(LiftSystem);
	
public:

	// ports to connect to users
	sc_in < Request > reqs_p[NUMBEROFUSERS];
	sc_event receive_req_e;

	// Contructor
	LiftSystem(sc_module_name name, unsigned int lifts, unsigned int floors);


	void GetRequests();

	void UpdateLifts();

	void ReceiveRequests();

	void AssignExternalRequest(int level, int up);

	void AssignInternalRequest(int lift, int level);

	// a lift's behavior
	void UpdateLiftStatus(int lift);

	void MovingLift(int lift);

	// get target floor to move for a lift
	int GetFloortoMove_Ext(int fromfloor, std::vector < int > targets, int direction);

	int GetFloortoMove_Int(int fromfloor, std::vector < int > targets);

	int ComputeMinimumDistance(int level, int up);

	// passby check
	void PassbyCheck();

private:
	// mutex for access shared data
	sc_mutex update_exreqs_up;
	sc_mutex update_exreqs_down;
	sc_mutex update_intreqs;
	sc_mutex update_liftstatus;

	// number of lifts
	unsigned int NumberofLifts;

	// number of floors
	unsigned int NumberofFloors;

	std::vector< int > ExternalRequestsUp;

	std::vector< int > ExternalRequestsDown;
	
	std::vector< std::vector< int > > InternalRequests;

	// model the lifts
	std::vector< lift_status_t > LiftStatus;

	// passby
	int passby;

};

#endif